package com.polopoly.modeshape;

import java.io.FileNotFoundException;

import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.management.RuntimeErrorException;

import org.infinispan.schematic.document.ParsingException;
import org.modeshape.common.collection.Problems;
import org.modeshape.jcr.ConfigurationException;
import org.modeshape.jcr.JcrRepository;
import org.modeshape.jcr.JcrSession;
import org.modeshape.jcr.ModeShapeEngine;
import org.modeshape.jcr.RepositoryConfiguration;

import com.google.inject.AbstractModule;
import com.google.inject.Provider;

public class ModeshapeModule extends AbstractModule {

    @Override
    protected void configure() {
        ModeShapeEngine engine = new ModeShapeEngine();
        engine.start();
        RepositoryConfiguration conf = readConf();
        JcrRepository repository = deployRepository(engine, conf);
        bind(JcrRepository.class).toInstance(repository);
        bind(Repository.class).toInstance(repository);
    }

    private JcrRepository deployRepository(ModeShapeEngine engine,
        RepositoryConfiguration conf) {
        try {
            return engine.deploy(conf);
        } catch (ConfigurationException e) {
            throw new RuntimeException(e);
        } catch (RepositoryException e) {
            throw new RuntimeException(e);
        }
    }

    private RepositoryConfiguration readConf() {
        RepositoryConfiguration conf;
        try {
            conf = RepositoryConfiguration.read("/repository.json");
            Problems problems = conf.validate();
            if (problems.hasErrors()) {
                System.err.println(problems);
                throw new RuntimeException("Couldn't start repository, see error log");
            }
        } catch (ParsingException e) {
            throw new RuntimeException(e);
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
        return conf;
    }

}
