package com.polopoly.modeshape;

import java.util.Iterator;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;

import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.infinispan.schematic.document.Json;

import com.sun.jersey.json.impl.provider.entity.JSONArrayProvider;

@Path("/")
public class ModeshapeService {
    private final Repository _repository;
    
    static AtomicInteger call = new AtomicInteger();

    @Inject
    public ModeshapeService(Repository repository) {
        _repository = repository;

    }

    @Path("document/{path}")
    @GET
    @Produces("application/json")
    public JSONObject getDocument(@PathParam("path") String path) throws Exception {
        Session session = _repository.login("main");
        Node rootNode = session.getRootNode();
        Node node = rootNode.getNode(path);

        JSONObject json = jsonify(node);

        return json;
    }

    @Path("document/{path}/count")
    @GET
    @Produces("application/json")
    public JSONObject count(@PathParam("path") String path)
        throws Exception
    {
        Session session = _repository.login("main");
        JSONObject json = new JSONObject();
        try {
            Node rootNode = session.getRootNode();
            Node pnode = rootNode.getNode(path);
            NodeIterator nodeIter = pnode.getNodes();
            long size = nodeIter.getSize();
            while (nodeIter.hasNext()) {
                Node inode = nodeIter.nextNode();
                System.out.println(inode.getPath());
                NodeIterator subNodeIter = inode.getNodes();
                size += subNodeIter.getSize();
                while (subNodeIter.hasNext()) {
                    Node tnode = subNodeIter.nextNode();
                    System.out.println(tnode.getPath());
                }
            }
            json.put("count", size);
        } finally {
            session.logout();
        }
        return json;
    }


    @Path("document/{path}")
    @PUT
    @Consumes("application/json")
    @Produces("text/plain")
    public String createDocument(@PathParam("path") String path, JSONObject json)
        throws Exception
    {
        Integer incr = call.incrementAndGet();
        long start = System.nanoTime();
        Session session = _repository.login("main");
        int i = 0;
        try {
            Node rootNode;
            if (!session.getRootNode().hasNode(path)) {
                rootNode = session.getRootNode().addNode(path);
            } else {
                rootNode = session.getRootNode().getNode(path);
            }
            Node numNode = rootNode.addNode(incr.toString());
            for (;i < 10; i++) {
                Node node = numNode.addNode(String.valueOf(i));
                //session.getRootNode().addNode(path + "/" + incr.toString() + "/" + i);
                node.setPrimaryType("nt:unstructured");
                writeJsonToNode(json, node);
            }
            session.save();
            //System.out.println("Doing PUT " + incr.toString());
        
        } finally {
            session.logout();
        }

        long end = System.nanoTime();
        return String.format("Creating 10 nodes took %dms", (end - start) / 1000000);
    }

    private void writeJsonToNode(JSONObject json, Node node)
        throws Exception {
        Iterator keys = json.keys();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            Object object = json.get(key);
            if (object instanceof JSONObject) {
                Node child = node.addNode(key);
                writeJsonToNode((JSONObject) object, child);
            }
            else if (object instanceof JSONArray) {
                JSONArray array = (JSONArray) object;
                for (int i = 0; i < array.length(); i++) {
                    Object item = array.get(i);
                    if (item instanceof JSONObject) {
                        Node itemNode = node.addNode(key);
                        writeJsonToNode((JSONObject) item, itemNode);
                    }
                    else {
                        // ToDo: create list
                        setProperty(node, key, item);
                    }
                }
            }
            else {
                setProperty(node, key, object);
            }
        }
    }

    private void setProperty(Node node, String key, Object object)
        throws Exception {
        if (object instanceof Long) {
            node.setProperty(key, (Long)object);
        } else if (object instanceof Integer) {
            node.setProperty(key, (Integer)object);
        } else if (object instanceof Double) {
            node.setProperty(key, (Double)object);
        } else if (object instanceof String) {
            node.setProperty(key, (String)object);
        } else if (object instanceof Boolean) {
            node.setProperty(key, (Boolean)object);
        } else {
            throw new IllegalArgumentException();
        }
    }

    private JSONObject jsonify(Node node)
        throws RepositoryException, JSONException
    {
        NodeIterator nodes = node.getNodes();
        JSONObject json = new JSONObject();
        while (nodes.hasNext()) {
            Node next = (Node) nodes.next();
            NodeIterator nodes2 = node.getNodes(next.getName());
            if (nodes2.getSize() > 1) {
                JSONArray array = new JSONArray();
                nodes.skip(nodes2.getSize() - 1);
                while (nodes2.hasNext()) {
                    array.put(jsonify(nodes2.nextNode()));
                }
                json.put(next.getName(), array);
            }
            else {
                json.put(next.getName(), jsonify(next));
            }
        }

        PropertyIterator properties = node.getProperties();
        while (properties.hasNext()) {
            Property property = (Property) properties.next();
            json.put(property.getName(), property.getString());
        }
        return json;
    }
}
